import logging

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from drf_yasg.utils import swagger_auto_schema

from rest_framework.parsers import JSONParser
from rest_framework.settings import api_settings

from purchase.views import ProcessTransactionBaseView
from purchase.models.choices import PurchaseResponseStatus
from purchase.serializers import PurchaseRequestSerialzier, PurchaseResponseSerializer
from purchase.controllers import PurchaseProcessController
from purchase.signals import purchase_completed

logger = logging.getLogger(__name__)


class ProcessPurchaseView(ProcessTransactionBaseView):
    parser_classes = [JSONParser]
    request_serializer = PurchaseRequestSerialzier
    response_serializer = PurchaseResponseSerializer

    @swagger_auto_schema(
        responses={200: response_serializer()},
        request_body=request_serializer(),
        operation_id="Process Purchase",
        tags=["Purchase"],
        operation_description=(
            "API to provide validating purchases from AppStore or GooglePlay.<br>"
            "Statuses:<br>"
            "1. ok - ok :)<br>"
            "2. purchase already created - purchase with given payload was already processed<br>"
            "3. data is not valid - provided data is not valid to create purchase<br>"
            "4. error - some error occurred, check logs"
        ),
    )
    def post(self, request, *args, **kwargs):
        return super().post(request, *args, **kwargs)

    @property
    def use_user(self):
        try:
            use_user = settings.PURCHASE_USER_ATTACHED
        except (ImproperlyConfigured, AttributeError):
            return False
        return use_user

    @property
    def permission_classes(self):
        if not self.use_user:
            return []
        return api_settings.DEFAULT_PERMISSION_CLASSES

    @property
    def authentication_classes(self):
        if not self.use_user:
            return []
        return api_settings.DEFAULT_AUTHENTICATION_CLASSES

    @property
    def status_choices(self):
        return PurchaseResponseStatus

    @property
    def controller_class(self):
        return PurchaseProcessController

    @property
    def signal(self):
        return purchase_completed



from django.db import models


class Platform(models.TextChoices):
    ios = "ios", "Ios"
    android = "android", "Android"


class PurchaseResponseStatus(models.TextChoices):
    ok = "ok"
    already_created = "purchase already created"
    data_is_not_valid = "data is not valid"
    error = "error"


class SubscriptionResponseStatus(models.TextChoices):
    ok = "ok"
    already_created = "subscription already created"
    data_is_not_valid = "data is not valid"
    error = "error"



import logging

from drf_yasg.utils import swagger_auto_schema

from rest_framework.parsers import JSONParser

from purchase.views import ProcessTransactionBaseView
from purchase.models.choices import SubscriptionResponseStatus
from purchase.serializers import PurchaseRequestSerialzier, SubscriptionResponseSerializer
from purchase.controllers import SubscriptionController
from purchase.signals import subscription_completed

logger = logging.getLogger(__name__)


class ProcessSubscriptionView(ProcessTransactionBaseView):
    parser_classes = [JSONParser]
    request_serializer = PurchaseRequestSerialzier
    response_serializer = SubscriptionResponseSerializer

    @swagger_auto_schema(
        responses={200: response_serializer()},
        request_body=request_serializer(),
        operation_id="Process Subscription",
        tags=["Subscription"],
        operation_description=(
            "API to provide validating subscriptions from AppStore or GooglePlay.<br>"
            "Statuses:<br>"
            "1. ok - ok :)<br>"
            "2. subscription already created - subscription with given payload was already processed<br>"
            "3. data is not valid - provided data is not valid to create purchase<br>"
            "4. error - some error occurred, check logs"
        ),
    )
    def post(self, request, *args, **kwargs):
        return super().post(request, *args, **kwargs)

    @property
    def status_choices(self):
        return SubscriptionResponseStatus

    @property
    def controller_class(self):
        return SubscriptionController

    @property
    def signal(self):
        return subscription_completed

    @property
    def use_user(self):
        return True



import time
import logging
import requests

from dataclasses import dataclass
from django.shortcuts import get_object_or_404

from purchase.models import Adjust
from purchase.strings.loggers import ADJUST_URL
from purchase.utils import repeatable_request

logger = logging.getLogger(__name__)


@dataclass
class AdjustLogger:
    fb: dict
    platform: str
    advertiser_id: str
    advertiser_key: str
    value_to_sum: float
    currency: str

    @property
    def credentials(self) -> Adjust:
        credentials = get_object_or_404(Adjust, platform=self.platform)
        return credentials

    @property
    def headers(self) -> dict:
        return {"Authorization": f"Bearer {self.credentials.authorization_token}"}

    def log_purchase(self) -> requests.Response:
        response = repeatable_request(
            url=ADJUST_URL, data=self.get_data(), headers=self.headers
        )
        return response

    def get_data(self) -> dict:
        return {
            "revenue": self.value_to_sum,
            "currency": self.currency,
            "event_token": self.credentials.purchase_event_token,
            "app_token": self.credentials.app_token,
            "s2s": 1,
            self.advertiser_key: self.advertiser_id,
            "created_at_unix": int(time.time()),
        }



import logging

from purchase.verifiers import AppleVerifier, GoogleVerifier
from purchase.models import Purchase
from purchase.strings.log_levels import (
    GOOGLE_ERROR_LEVEL,
    APPLE_ERROR_LEVEL,
)
from purchase.controllers import ProcessControllerABC

logger = logging.getLogger(__name__)


class PurchaseProcessController(ProcessControllerABC):
    model = Purchase
    logging_enabled = True

    def apple_verify(self) -> (bool, bool):
        is_sandbox = self.is_sandbox
        result = False
        try:
            apple_verifier = AppleVerifier(
                receipt=self.receipt_data,
                is_sandbox=self.is_sandbox,
                product_id=self.serializer_data["data"]["product_id"],
                platform=self.platform,
                version=self.version,
                transaction_id=self.transaction_id,
            )
            is_sandbox, result = apple_verifier.verify()
        except Exception as err:
            details = {
                "transaction_id": self.transaction_id,
                "receipt": self.receipt_data,
            }
            self.lc.save_error_log(
                error_message=str(err), log_level=APPLE_ERROR_LEVEL, details=details
            )
        return is_sandbox, result

    def google_verify(self) -> (bool, bool):
        result = False
        try:
            google_verifier = GoogleVerifier(
                receipt=self.receipt_data,
                platform=self.platform,
                version=self.version,
            )
            result = google_verifier.verify()
        except Exception as err:
            details = {
                "transaction_id": self.transaction_id,
                "receipt": self.receipt_data,
            }
            self.lc.save_error_log(
                error_message=str(err), log_level=GOOGLE_ERROR_LEVEL, details=details
            )
        return self.is_sandbox, result



from .log import Log
from .config import Config
from .google import Google
from .adjust import Adjust
from .facebook import Facebook
from .purchase import Purchase
from .adjust_log import AdjustLog
from .apps_flyer import AppsFlyer
from .subscription import Subscription



import logging

from django.db import models
from django.utils import timezone
from django.conf import settings

from purchase.models.choices import Platform
from purchase.verifiers import AppleVerifier, GoogleVerifier

logger = logging.getLogger(__name__)


class Subscription(models.Model):
    user = models.ForeignKey(to=settings.USER_MODEL, on_delete=models.SET_NULL, null=True)

    currency = models.CharField(max_length=25)
    log_time = models.CharField(max_length=11)
    ext_info = models.CharField(max_length=255, null=True)
    product_id = models.CharField(max_length=255)
    transaction_id = models.CharField(max_length=255)
    bundle_short_version = models.CharField(max_length=255)
    fb_user_id = models.CharField(max_length=255, null=True)
    advertiser_id = models.CharField(max_length=255, null=True)
    platform = models.CharField(max_length=8, choices=Platform.choices)

    value_to_sum = models.FloatField()

    body = models.JSONField()

    is_sandbox = models.BooleanField(default=False)

    active_till = models.DateTimeField(null=True)
    google_token = models.CharField(null=True, max_length=8192, help_text="Only for android platform")
    _is_active = models.DateField(null=True)
    _expired = models.BooleanField(default=False)

    def __verify(self):
        try:
            if self.platform == Platform.ios:
                return AppleVerifier(
                    receipt=self.body["data"]["receipt_data"],
                    is_sandbox=self.is_sandbox,
                    product_id=self.product_id,
                    platform=self.platform,
                    version=self.body["data"]["fb"]["bundle_short_version"],
                    transaction_id=self.transaction_id,
                    is_sub=True,
                )
            return self.is_sandbox, GoogleVerifier(
                receipt=self.body["data"]["receipt_data"],
                platform=self.platform,
                version=self.body["data"]["fb"]["bundle_short_version"],
                is_sub=True,
            )
        except Exception as err:
            logger.error(str(err))
            return False, False

    @property
    def is_active(self):
        if self._expired:
            return False
        today = timezone.now().today()
        if today == self._is_active:
            return True
        _, active = self.__verify()
        if not active:
            self._expired = True
        self._is_active = today
        self.save()
        return not self._expired

    def set_transaction_id_to_fake(self):
        self.transaction_id = f"fake_{self.transaction_id}"
        self.save()



import logging

from rest_framework import serializers

from purchase.models.choices import Platform


logger = logging.getLogger(__name__)


class FbSerializer(serializers.Serializer):
    """fb field of ChargeDataSerializer"""

    user_id = serializers.CharField(max_length=500, required=True, allow_null=True)
    advertiser_tracking_enabled = serializers.BooleanField(required=True)
    application_tracking_enabled = serializers.BooleanField(required=True)
    product_quantity = serializers.IntegerField(required=True)
    product_title = serializers.CharField(required=True)
    product_description = serializers.CharField(required=True)
    num_items = serializers.IntegerField(required=True)
    transaction_date = serializers.CharField(max_length=40, required=True)
    extinfo = serializers.ListField(
        child=serializers.CharField(allow_blank=True), required=True
    )


class ReceiptSerializer(serializers.Serializer):
    """receipt_data field of ChargeDataSerializer"""

    store = serializers.CharField(required=True)
    transaction_id = serializers.CharField(required=True)
    payload = serializers.CharField(required=True)


class PurchaseDataSerializer(serializers.Serializer):
    """data field of ChargeRequestSerializer"""

    receipt_data = ReceiptSerializer(required=True)
    appsflyer_id = serializers.CharField(max_length=100, required=False)
    adjust_device_id = serializers.CharField(max_length=255, required=False)
    product_id = serializers.CharField(required=True)
    advertiser_id = serializers.CharField(required=True, allow_null=True)
    value_to_sum = serializers.FloatField(required=True)
    currency = serializers.CharField(required=True, max_length=18)
    bundle_short_version = serializers.CharField(max_length=10, required=True, allow_null=True)
    log_time = serializers.IntegerField(required=True)
    fb = FbSerializer(required=False)


class PurchaseRequestSerialzier(serializers.Serializer):
    platform = serializers.ChoiceField(
        choices=Platform.choices, required=True, write_only=True
    )
    is_sandbox = serializers.BooleanField(required=True, write_only=True)
    data = PurchaseDataSerializer(required=True, write_only=True)



# Generated by Django 3.1.6 on 2021-09-14 13:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Adjust",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("game", models.CharField(max_length=50)),
                ("app_token", models.CharField(max_length=50)),
                ("authorization_token", models.CharField(max_length=50)),
                ("purchase_event_token", models.CharField(max_length=50)),
                (
                    "platform",
                    models.CharField(
                        choices=[("ios", "Ios"), ("android", "Android")], max_length=20
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Adjust",
            },
        ),
        migrations.CreateModel(
            name="AdjustLog",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("request_url", models.URLField()),
                ("response", models.JSONField()),
                ("request_data", models.JSONField()),
                ("request_headers", models.JSONField()),
                ("created_at", models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name="AppsFlyer",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("game", models.CharField(max_length=50)),
                ("app_id", models.CharField(max_length=100)),
                ("dev_key", models.CharField(max_length=300)),
                (
                    "platform",
                    models.CharField(
                        choices=[("ios", "Ios"), ("android", "Android")], max_length=20
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "AppsFlyer",
            },
        ),
        migrations.CreateModel(
            name="Config",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("game", models.CharField(max_length=50)),
                (
                    "platform",
                    models.CharField(
                        choices=[("ios", "Ios"), ("android", "Android")], max_length=20
                    ),
                ),
                ("is_fb_log_enabled", models.BooleanField(default=True)),
                ("is_af_log_enabled", models.BooleanField(default=True)),
                ("is_adjust_log_enabled", models.BooleanField(default=True)),
            ],
        ),
        migrations.CreateModel(
            name="Facebook",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("game", models.CharField(max_length=50)),
                ("app_id", models.CharField(max_length=100)),
                ("client_secret", models.CharField(max_length=100)),
                (
                    "platform",
                    models.CharField(
                        choices=[("ios", "Ios"), ("android", "Android")], max_length=20
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Facebook",
            },
        ),
        migrations.CreateModel(
            name="Google",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("game", models.CharField(max_length=50)),
                ("client_id", models.CharField(max_length=50)),
                ("project_id", models.CharField(max_length=50)),
                ("private_key_id", models.CharField(max_length=60)),
                ("type", models.CharField(default="service_account", max_length=20)),
                ("private_key", models.TextField()),
                ("client_email", models.EmailField(max_length=254)),
                ("auth_uri", models.URLField()),
                ("token_uri", models.URLField()),
                ("client_x509_cert_url", models.URLField()),
                ("auth_provider_x509_cert_url", models.URLField()),
            ],
            options={
                "verbose_name_plural": "Google",
            },
        ),
        migrations.CreateModel(
            name="Log",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("game", models.CharField(max_length=50, null=True)),
                ("version", models.CharField(max_length=10, null=True)),
                ("log_level", models.CharField(blank=True, max_length=15, null=True)),
                (
                    "platform",
                    models.CharField(
                        choices=[("ios", "Ios"), ("android", "Android")], max_length=20
                    ),
                ),
                ("message", models.TextField()),
                ("details", models.JSONField(blank=True, null=True)),
                ("time", models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name="Purchase",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("game", models.CharField(max_length=50)),
                ("currency", models.CharField(max_length=25)),
                ("log_time", models.CharField(max_length=11)),
                ("ext_info", models.CharField(max_length=255)),
                ("product_id", models.CharField(max_length=255)),
                ("transaction_id", models.CharField(max_length=255)),
                ("bundle_short_version", models.CharField(max_length=255)),
                ("fb_user_id", models.CharField(max_length=255, null=True)),
                ("advertiser_id", models.CharField(max_length=255, null=True)),
                (
                    "platform",
                    models.CharField(
                        choices=[("ios", "Ios"), ("android", "Android")], max_length=8
                    ),
                ),
                ("value_to_sum", models.FloatField()),
                ("body", models.JSONField()),
                ("is_valid", models.BooleanField(default=False)),
                ("is_sandbox", models.BooleanField(default=False)),
                ("fb_is_logged", models.BooleanField(default=False)),
                ("af_is_logged", models.BooleanField(default=False)),
                ("adjust_is_logged", models.BooleanField(default=False)),
            ],
        ),
        migrations.AddConstraint(
            model_name="google",
            constraint=models.UniqueConstraint(fields=("game",), name="unique"),
        ),
        migrations.AddField(
            model_name="adjustlog",
            name="purchase",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="adjust_log",
                to="purchase.purchase",
            ),
        ),
    ]



GOOGLE_MODEL_NOT_FOUND = "Google model not found"

# Purchase process errors
APPSFLYER_ID_NOT_FOUND = "appsflyer_id not found in data"
ADJUST_ADVERTISER_NOT_FOUND = "Adjust advertiser_id not found"

ALL_ZEROS = "00000000-0000-0000-0000-000000000000"



import logging
import requests

from datetime import datetime
from dataclasses import dataclass
from django.shortcuts import get_object_or_404

from purchase.models import Facebook
from purchase.utils import repeatable_request
from purchase.models.facebook import GRAPH_API_VERSION

logger = logging.getLogger(__name__)


@dataclass
class FacebookLogger:
    platform: str
    transaction_id: str
    fb_data: dict
    advertiser_id: str
    bundle_short_version: str
    product_id: str
    value_to_sum: float
    currency: str

    @property
    def _credentials(self):
        credentials = get_object_or_404(
            Facebook, platform=self.platform
        )
        return credentials

    def log_purchase(self) -> requests.Response:
        response = repeatable_request(
            url=self._get_url(), json=self._get_data(), headers=self._get_headers()
        )
        return response

    def _get_data(self) -> dict:
        return {
            "event": "CUSTOM_APP_EVENTS",
            "advertiser_id": self.advertiser_id,
            "bundle_version": self.bundle_short_version,
            "bundle_short_version": self.bundle_short_version,
            "app_user_id": self.fb_data["user_id"],
            "advertiser_tracking_enabled": str(
                int(self.fb_data["advertiser_tracking_enabled"])
            ),
            "application_tracking_enabled": "1",
            "extinfo": self.fb_data["extinfo"],
            "custom_events": [
                {
                    "_logTime": int(datetime.now().timestamp()),
                    "fb_transaction_date": self.fb_data["transaction_date"],
                    "Transaction Identifier": self.transaction_id,
                    "fb_content": [{"id": self.product_id, "quantity": 1}],
                    "_valueToSum": self.value_to_sum,
                    "fb_currency": self.currency,
                    "Product Title": self.fb_data["product_title"],
                    "fb_num_items": 1,
                    "fb_content_type": "product",
                    "fb_iap_product_type": "inapp",
                    "_eventName": "fb_mobile_purchase",
                }
            ],
        }

    def _get_headers(self) -> dict:
        return {"Authorization": f"Bearer {self._credentials.app_access_token}"}

    def _get_url(self) -> str:
        return f"https://graph.facebook.com/{GRAPH_API_VERSION}/{self._credentials.app_id}/activities"



import logging

from purchase.models import Subscription
from purchase.verifiers import AppleVerifier, GoogleVerifier
from purchase.controllers import ProcessControllerABC


logger = logging.getLogger(__name__)


class SubscriptionController(ProcessControllerABC):
    model = Subscription
    logging_enabled = False

    def apple_verify(self) -> (bool, bool):
        is_sandbox = self.is_sandbox
        result = False
        try:
            apple_verifier = AppleVerifier(
                receipt=self.receipt_data,
                is_sandbox=self.is_sandbox,
                product_id=self.serializer_data["data"]["product_id"],
                platform=self.platform,
                version=self.version,
                transaction_id=self.transaction_id,
                is_sub=True,
            )
            is_sandbox, result = apple_verifier.verify()
        except Exception as err:
            logger.error(str(err))
        return is_sandbox, result

    def google_verify(self) -> (bool, bool):
        result = False
        try:
            google_verifier = GoogleVerifier(
                receipt=self.receipt_data,
                platform=self.platform,
                version=self.version,
                is_sub=True,
            )
            result = google_verifier.verify()
        except Exception as err:
            logger.error(str(err))
        return self.is_sandbox, result



from django.db import models
from django.conf import settings

from purchase.models.choices import Platform


class Purchase(models.Model):
    """Model of Purchases from games"""
    user = models.ForeignKey(to=settings.USER_MODEL, on_delete=models.SET_NULL, null=True)

    currency = models.CharField(max_length=25)
    log_time = models.CharField(max_length=11)
    ext_info = models.CharField(max_length=255, null=True)
    product_id = models.CharField(max_length=255)
    transaction_id = models.CharField(max_length=255)
    bundle_short_version = models.CharField(max_length=255)
    fb_user_id = models.CharField(max_length=255, null=True)
    advertiser_id = models.CharField(max_length=255, null=True)
    platform = models.CharField(max_length=8, choices=Platform.choices)

    value_to_sum = models.FloatField()

    body = models.JSONField()

    is_valid = models.BooleanField(default=False)
    is_sandbox = models.BooleanField(default=False)
    fb_is_logged = models.BooleanField(default=False)
    af_is_logged = models.BooleanField(default=False)
    adjust_is_logged = models.BooleanField(default=False)

    def set_transaction_id_to_fake(self):
        self.transaction_id = f"fake_{self.transaction_id}"
        self.save()



PURCHASE_CREATE = "Purchase create"
GOOGLE_ERROR_LEVEL = "Google"
APPLE_ERROR_LEVEL = "Apple"
FACEBOOK_LOGGING_ERROR_LEVEL = "Facebook"
ADJUST_ERROR_LEVEL = "Adjust"
APPS_FLYER_ERROR_LEVEL = "Apps Flyer"



from django.db import models

from purchase.models.choices import Platform

GRAPH_API_VERSION = "v9.0"


class Facebook(models.Model):
    app_id = models.CharField(max_length=100)
    client_secret = models.CharField(max_length=100)
    platform = models.CharField(choices=Platform.choices, max_length=20, unique=True)

    @property
    def app_access_token(self) -> str:
        """Get app fb access token as access_token={app-id}|{client-token}"""
        return f"{self.app_id}|{self.client_secret}"

    class Meta:
        verbose_name_plural = "Facebook"



# Google verify
GOOGLE_AUTH_URL = "https://www.googleapis.com/auth/androidpublisher"

# Apple verify
APPLE_URL = "https://buy.itunes.apple.com/verifyReceipt"
APPLE_SANDBOX_URL = "https://sandbox.itunes.apple.com/verifyReceipt"
APPLE_HEADERS = {"Content-Type": "application/x-www-form-urlencoded"}



import logging

from purchase.models import Log, Purchase, AdjustLog, Config
from purchase.models.choices import Platform
from purchase.strings.loggers import ADJUST_URL
from purchase.loggers import AdjustLogger, FacebookLogger, AppsFlyerLogger
from purchase.strings.errors import APPSFLYER_ID_NOT_FOUND, ALL_ZEROS, ADJUST_ADVERTISER_NOT_FOUND
from purchase.strings.log_levels import (
    FACEBOOK_LOGGING_ERROR_LEVEL,
    ADJUST_ERROR_LEVEL,
    APPS_FLYER_ERROR_LEVEL,
)

logger = logging.getLogger(__name__)


class LogController:
    def __init__(self, platform, version, fb, transaction_id, serializer_data):
        self.platform = platform
        self.version = version
        self.fb = fb
        self.transaction_id = transaction_id
        self.serializer_data = serializer_data

    @property
    def appsflyer_id(self):
        if (
            "appsflyer_id" not in self.serializer_data["data"]
            or self.serializer_data["data"]["appsflyer_id"] is None
        ):  # pragma: no cover
            raise ValueError(APPSFLYER_ID_NOT_FOUND)
        return self.serializer_data["data"]["appsflyer_id"]

    def get_advertiser_id(self):
        advertiser_id = self.serializer_data["data"]["advertiser_id"]
        if advertiser_id is None or advertiser_id == ALL_ZEROS or advertiser_id == "":
            if "adjust_device_id" not in self.serializer_data["data"]:
                raise ValueError(ADJUST_ADVERTISER_NOT_FOUND)
            return "adid", self.serializer_data["data"]["adjust_device_id"]
        if self.platform == Platform.android:
            return "gps_adid", advertiser_id
        return "idfa", advertiser_id

    def get_details_for_log(self):
        return {"transaction_id": self.transaction_id, "data": self.serializer_data}

    def save_error_log(self, error_message: str, log_level: str, details: dict):
        Log.objects.create(
            platform=self.platform,
            version=self.version,
            log_level=log_level,
            message=error_message,
            details=details,
        )

    def handle_logger(self,
                      purchase_obj: Purchase,
                      out_logger: FacebookLogger or AppsFlyerLogger or AdjustLogger,
                      error_level,
                      log_field: str) -> str:
        response = out_logger.log_purchase()
        if response.status_code != 200:
            err = f"api error: request error [{response.url}] {response.status_code = }, {response.text = }"
            self.save_error_log(
                error_message=err,
                log_level=error_level,
                details=self.get_details_for_log(),
            )
        else:
            setattr(purchase_obj, log_field, True)
            purchase_obj.save()
        return response.text

    def log_in_facebook(self, purchase_obj: Purchase):
        fb_logger = FacebookLogger(
            platform=self.platform,
            transaction_id=self.transaction_id,
            fb_data=self.fb,
            advertiser_id=self.serializer_data["data"]["advertiser_id"],
            bundle_short_version=self.version,
            product_id=self.serializer_data["data"]["product_id"],
            value_to_sum=self.serializer_data["data"]["value_to_sum"],
            currency=self.serializer_data["data"]["currency"],
        )
        self.handle_logger(purchase_obj, fb_logger, FACEBOOK_LOGGING_ERROR_LEVEL, "fb_is_logged")

    def log_in_appsflyer(self, purchase_obj: Purchase):
        appsflyer_logger = AppsFlyerLogger(
            platform=self.platform,
            appsflyer_id=self.appsflyer_id,
            fb=self.fb,
            bundle_short_version=self.version,
            currency=self.serializer_data["data"]["currency"],
            value_to_sum=self.serializer_data["data"]["value_to_sum"],
            product_id=self.serializer_data["data"]["product_id"],
            advertiser_id=self.serializer_data["data"]["advertiser_id"],
            log_time=self.serializer_data["data"]["log_time"]
        )
        self.handle_logger(purchase_obj, appsflyer_logger, APPS_FLYER_ERROR_LEVEL, "af_is_logged")

    def log_in_adjust(self, purchase_obj: Purchase):
        advertiser_key, advertiser_id = self.get_advertiser_id()
        adjust_logger = AdjustLogger(
            fb=self.fb,
            platform=self.platform,
            advertiser_id=advertiser_id,
            advertiser_key=advertiser_key,
            value_to_sum=self.serializer_data["data"]["value_to_sum"],
            currency=self.serializer_data["data"]["currency"],
        )
        text = self.handle_logger(purchase_obj, adjust_logger, ADJUST_ERROR_LEVEL, "adjust_is_logged")
        AdjustLog.objects.get_or_create(
            purchase=purchase_obj,
            request_url=ADJUST_URL,
            response=text,
            request_data=adjust_logger.get_data(),
            request_headers=adjust_logger.headers,
        )

    def get_enable_loggers(self):
        config = Config.objects.filter(platform=self.platform)
        loggers = [
            self.log_in_adjust if not config.exists() or config.first().is_adjust_log_enabled else None,
            self.log_in_appsflyer if not config.exists() or config.first().is_af_log_enabled else None,
            self.log_in_facebook if not config.exists() or config.first().is_fb_log_enabled else None,
        ]
        return loggers

    def log(self, purchase_obj: Purchase):
        enabled_loggers = self.get_enable_loggers()
        for out_logger in enabled_loggers:
            if out_logger is None:
                continue
            out_logger(purchase_obj=purchase_obj)



from rest_framework import serializers

from purchase.models.choices import SubscriptionResponseStatus


class SubscriptionResponseSerializer(serializers.Serializer):
    status = serializers.ChoiceField(
        choices=SubscriptionResponseStatus.choices,
        required=True,
    )
    error = serializers.CharField(required=False)



from purchase.models import Config
from rest_framework import serializers


class ConfigResponseSerializer(serializers.ModelSerializer):
    fb = serializers.BooleanField(required=True, source="is_fb_log_enabled")
    apps_flyer = serializers.BooleanField(required=True, source="is_af_log_enabled")
    adjust = serializers.BooleanField(required=True, source="is_adjust_log_enabled")

    class Meta:
        model = Config
        fields = ("fb", "apps_flyer", "adjust")



# Generated by Django 3.2.7 on 2021-09-20 10:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("purchase", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="adjust",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="adjustlog",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="appsflyer",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="config",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="facebook",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="google",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="log",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="purchase",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]



# Generated by Django 3.2.7 on 2022-07-28 04:19

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.USER_MODEL),
        ('purchase', '0004_subscription'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subscription',
            name='user',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.USER_MODEL),
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['purchase',
 'purchase.controllers',
 'purchase.exceptions',
 'purchase.loggers',
 'purchase.migrations',
 'purchase.models',
 'purchase.serializers',
 'purchase.signals',
 'purchase.strings',
 'purchase.templates',
 'purchase.verifiers',
 'purchase.views']

package_data = \
{'': ['*']}

install_requires = \
['Django>=4.0.1,<5.0.0',
 'Pygments>=2.10.0,<3.0.0',
 'django-admin-rangefilter>=0.8.1,<0.9.0',
 'django-filter>=2.4.0,<3.0.0',
 'djangorestframework>=3.12.4,<4.0.0',
 'drf-yasg>=1.21.3,<2.0.0',
 'google-api-python-client>=2.21.0,<3.0.0',
 'google-auth-oauthlib>=0.4.6,<0.5.0',
 'google-auth>=2.1.0,<3.0.0',
 'google>=3.0.0,<4.0.0',
 'requests>=2.26.0,<3.0.0']

setup_kwargs = {
    'name': 'django-purchase-core',
    'version': '0.3.3',
    'description': 'A reusable Django app for creating, logging and verifying purchases.',
    'long_description': 'Purchase Core\n===============\n\nA reusable Django app for creating, logging and verifying purchases.\n\nQuick start\n-----------\n\n1. Install Django Purchase Core & Dependencies:\n\n    >>> pip install django-purchase-core\n\n\n2. Add "purchase", "rest_framework\', and "rangefilter" to your INSTALLED_APPS setting like this:\n\n.. code:: python\n\n        INSTALLED_APPS = [\n            ...,\n            \'rest_framework\',\n            \'purchase\',\n            \'rangefilter\',\n            ...,\n        ]\n\n3. Add the following to app_config.urls:\n\n.. code:: python\n\n    from django.conf.urls import url, include\n\n    urlpatterns = [\n        ...,\n        path("api/", include("purchase.urls")),\n        ...,\n    ]\n\n\n4. Run Django Commands:\n\n    >>> python manage.py makemigrations\n    >>> python manage.py migrate\n\n\n5. Configure configuration and credentials for your game in the admin panel.\n\nAdd progress level update processing\n-------------------------------------\n\n1. To work with subscription you first need to set authorization and user model:\n\n   * setup "PURCHASE_USER_ATTACHED" in you settings.py to True (not provided equals False, when it False no signal will be called, user model wont be used)\n\n   .. code:: python\n\n        ...\n        PURCHASE_USER_ATTACHED = True\n        ...\n\n   * configure user model (in your settings.py)\n\n   .. code:: python\n\n        USER_MODEL = YourUserModel\n\n   * configure auth and permission classes for DRF (in your settings.py)\n\n   .. code:: python\n\n        REST_FRAMEWORK = {\n            "DEFAULT_PERMISSION_CLASSES": [\n                "rest_framework.permissions.IsAuthenticated"\n            ],\n            "DEFAULT_AUTHENTICATION_CLASSES": [\n                "path.to.your.auth.class.or.base"\n            ],\n        }\n\n   * to handle completed purchase setup receiver to update progress, which will receive "instance"\n\n   .. code:: python\n\n            from django.dispatch import receiver\n\n            from purchase.signals import purchase_completed\n\n            @receiver(purchase_completed)\n            def purchase_completed(sender, **kwargs):\n                purchase = kwargs["instance"]\n                user = purchase.user  # User completed purchase\n                purchase_id = purchase.purchase_id  # Your product ID, as it presented in store\n                ...\n\n\n2. To work with subscription you first need to set authorization and user model:\n\n   * configure user model, auth and permission classes for DRF (in your settings.py, as for purchase with user)\n\n   .. code:: python\n\n        ...\n        USER_MODEL = YourUserModel\n        ...\n\n        ...\n        REST_FRAMEWORK = {\n            ...\n            "DEFAULT_PERMISSION_CLASSES": [\n                "rest_framework.permissions.IsAuthenticated"\n            ],\n            ...\n            "DEFAULT_AUTHENTICATION_CLASSES": [\n                "path.to.your.auth.class.or.base"\n            ],\n            ...\n        }\n        ...\n\n   * setup receiver for signal, which will receive "instance" as Subscription model instance\n\n   .. code:: python\n\n        from django.dispatch import receiver\n\n        from purchase.signals import subscription_completed\n\n        @receiver(subscription_completed)\n        def subscription_completed(sender, **kwargs):\n            subscription = kwargs["instance"]\n            user = subscription.user  # User completed subscription\n            subscription_id = subscription.product_id  # your subscription ID, as it presented in store\n            ...\n',
    'author': 'Qmobi',
    'author_email': 'info@qmobi.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/boy-scouts/game-core-purchase',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from rest_framework.parsers import JSONParser
from django.shortcuts import get_object_or_404
from rest_framework.generics import RetrieveAPIView
from django_filters import rest_framework as filters
from django_filters.rest_framework import DjangoFilterBackend

from purchase.models import Config
from purchase.serializers.config_response_serializer import ConfigResponseSerializer


class MonthlyPredictionFilter(filters.FilterSet):
    platform = filters.CharFilter(method="platform_filter", required=True)

    def platform_filter(self, _queryset, _name, value):
        self.queryset = self.queryset.filter(platform=value)
        return self.queryset

    class Meta:
        model = Config
        fields = ("platform", )


class LogConfig(RetrieveAPIView):
    filter_backends = (DjangoFilterBackend,)
    filterset_class = MonthlyPredictionFilter
    queryset = Config.objects.all()
    serializer_class = ConfigResponseSerializer
    parser_classes = [JSONParser]
    authentication_classes = []
    permission_classes = []

    def get(self, request, *args, **kwargs):
        return super().get(request, *args, **kwargs)

    def get_object(self):
        queryset = self.filter_queryset(self.get_queryset())
        config = get_object_or_404(queryset)
        return config
